######################################################################################
######################################################################################

# R-code to compare BUGS / JAGS / STAN on a simple linear regression

# E. RIVOT & E. PARENT - June 2014

######################################################################################
######################################################################################


rm(list = ls())



##################################################################
#                          bugs
##################################################################

library(BRugs)
library(coda)

beg.bugs <- proc.time()

model.bugs <- "model{

# prior

a ~ dnorm(0,0.0001)
b ~ dnorm(0,0.0001)
sigma ~ dunif(0,100)
tau <- 1/(sigma*sigma)

b.2 <- b - a*mean(Ta[])

# likelihood

for (i in 1:n)
{
mean.Tw[i] <- a*(Ta[i]-mean(Ta[])) + b
Tw[i] ~ dnorm(mean.Tw[i],tau)
}

}"

cat(model.bugs,file="model.bugs.txt")

n.burnIn <- 1000
n.iter <- 2000
thin <- 1
n.Chains <- 3

model<-"model.bugs.txt"
data <- "data.txt"
inits <- "inits.txt"

modelCheck(model) 
modelData(data) 
modelCompile(numChains=n.Chains) 
modelInits(c(inits,inits,inits))
modelGenInits()

monitor <- c("a", "b.2", "sigma")

modelUpdate(numUpdates = n.burnIn, thin = 1) 
samplesSet(monitor) 
modelUpdate(numUpdates = n.iter, thin = thin) 

end.bugs <- proc.time()
time.bugs <- end.bugs - beg.bugs

mcmc.bugs <- as.data.frame(as.matrix(buildMCMC("*")))




##################################################################
#                          jags
##################################################################

library(rjags)
library(coda)

beg.jags <- proc.time()

data <- source("data.txt")$value

model.jags <- "

# data block

data{
Ta.std <- Ta-mean(Ta[])
}

model{

# prior

a ~ dnorm(0,0.0001)
b ~ dnorm(0,0.0001)
sigma ~ dunif(0,100)
tau <- 1/(sigma*sigma)

b.2 <- b - a*mean(Ta[])

# likelihood

for (i in 1:n)
{
Tw[i] ~ dnorm(a*Ta.std[i] + b,tau)
}

}"

cat(model.jags,file="model.jags.txt")

monitor <- c("a", "b.2", "sigma")

# Compile the model and adapt the MCMC samplers

n.chains = 3
n.adapt = 1000
reg.lin.jags <- jags.model("model.jags.txt",data=data,n.chain=n.chains,n.adapt=n.adapt)

# Run the model, record the computer time needed to run the model

thin = 1
samples = 2000
mcmc.jags <- coda.samples(reg.lin.jags,monitor,n.iter=samples*thin,thin=thin)

mcmc.jags <- as.data.frame(as.matrix(mcmc.jags[,c("a","b.2","sigma")]))

end.jags <- proc.time()
time.jags <- end.jags - beg.jags




##################################################################
#                            R stan
##################################################################

library(rstan)

beg.stan <- proc.time()

# model

model.stan <- "

data
{
int<lower=0> n; // number of data to consider
real Tw[n]; // water temperature
real Ta[n]; // air temperature
}

parameters
{
real a;
real b;
real<lower=0> sigma;
}

model{

# Prior
a ~ normal(0,100);
b ~ normal(0,100);
sigma ~ uniform(0,100); // Normal distributions use sigma (and not tau)

# likelihood

for (i in 1:n)
{
real mu ;    // possible to use an internal variable
mu <- a*Ta[i] + b ; 
Tw[i] ~ normal(mu,sigma) ; 
}

} # end model

" # end code



# Fit
# -----------------------

fit <- stan(
model_code = model.stan, 
data = data,
iter = 3000, 
chains = 3,
warmup = 1000,
pars=c("a","b","sigma")
)

end.stan <- proc.time()
time.stan <- end.stan - beg.stan


# outputs
# -----------------------

mcmc.stan <- as.data.frame(fit, pars=c("a","b","sigma"))



##################################################################
#                       Compare results
##################################################################

time.bugs
time.jags
time.stan


# marginal posterior
# -----------------------------------
windows()
par(mfrow=c(1,3))
boxplot(mcmc.bugs, outline=FALSE)
boxplot(mcmc.jags, outline=FALSE)
boxplot(mcmc.stan, outline=FALSE)


# Joint mcmc sample
# -----------------------------------

windows()
par(mfrow=c(1,3))

n.plot1 <- 1
n.plot2 <- 50
plot(mcmc.bugs$a[n.plot1:n.plot2],mcmc.jags$b.2[n.plot1:n.plot2], xlim = c(0.3,0.7), ylim=c(6,10), type = "l", main = "BUGS")
plot(mcmc.jags$a[n.plot1:n.plot2],mcmc.jags$b.2[n.plot1:n.plot2], xlim = c(0.3,0.7), ylim=c(6,10), type = "l", main = "JAGS")
plot(mcmc.stan$a[n.plot1:n.plot2],mcmc.stan$b[n.plot1:n.plot2], xlim = c(0.3,0.7), ylim=c(6,10), type = "l", main = "STAN")


# Marginal posterior
# -----------------------------------

x <- mcmc.bugs$a
y <- mcmc.jags$a
z <- mcmc.stan$a

windows()
cise.text <- 1.5
par(mfrow = c(1,1), oma = c(1,1,1,1))

plot(x=0, y=-1, ylim = c(0,max(density(x)$y)*1.2), xlim = c(0.3,0.7), ylab = "", xlab = "", bty="n")
lines(density(x), col = "black", lty = 1, lwd = 2)
lines(density(y), col = "black", lty = 2, lwd = 2)
lines(density(z), col = "black", lty = 3, lwd = 2)

mtext(side=3, text = expression(paste("Marginal posterior of ",a)), bty="n", line=1, cex=1.5, font = 1)
mtext(side=1, text = "a", bty="n", line=3, cex=cise.text, font = 1)
mtext(side=2, text = "Density", bty="n", line=3, cex=cise.text, font = 1)

legend(	legend = c("BUGS","JAGS","STAN"),
 		col = c("black","black","black"), 	
 		lty = c(1,2,3), lwd = c(2,2,2), x = "topright", cex = 1, bty ="n") 


